#include "../std.h"

class CheckSum {
 public:
  CheckSum(void) { };
  virtual ~CheckSum(void) { };
  virtual void start(void) { };
  virtual void add(void* buf,unsigned long long int len) { };
  virtual void end(void) { };
  virtual void result(unsigned char*& res,unsigned int& len) const { len=0; };
  virtual int print(char* buf,int len) const { if(len>0) buf[0]=0; return 0; };
};


class CRC32Sum: public CheckSum {
 private:
  uint32_t r;
  unsigned long long count;
  bool computed;
 public:
  CRC32Sum(void);
  virtual ~CRC32Sum(void) { };
  virtual void start(void);
  virtual void add(void* buf,unsigned long long int len);
  virtual void end(void);
  virtual void result(unsigned char*& res,unsigned int& len) const { res=(unsigned char*)&r; len=4; };
  virtual int print(char* buf,int len) const;
  uint32_t crc(void) const { return r; };
};

class MD5Sum: public CheckSum {
 private:
  bool computed;
  uint32_t A;
  uint32_t B;
  uint32_t C;
  uint32_t D;
  uint64_t count;
  uint32_t X[16];
  uint Xlen;
  // uint32_t T[64];
 public:
  MD5Sum(void);
  virtual void start(void);
  virtual void add(void* buf,unsigned long long int len);
  virtual void end(void);
  virtual void result(unsigned char*& res,unsigned int& len) const { res=(unsigned char*)&A; len=16; };
  virtual int print(char* buf,int len) const;
};

class CheckSumAny: public CheckSum {
  CheckSum* cs;
 public:
  CheckSumAny(CheckSum* c = NULL):cs(c) { };
  CheckSumAny(const char* type):cs(NULL) {
    if(!type) return;
    if(strncasecmp("cksum",type,5) == 0) { cs=new CRC32Sum; return; };
    if(strncasecmp("md5",type,3) == 0) { cs=new MD5Sum; return; };
  };
  virtual ~CheckSumAny(void) { if(cs) delete cs; };
  virtual void start(void) { if(cs) cs->start(); };
  virtual void add(void* buf,unsigned long long int len) {
    if(cs) cs->add(buf,len);
  };
  virtual void end(void) { if(cs) cs->end(); };
  virtual void result(unsigned char*& res,unsigned int& len) const {
    if(cs) { cs->result(res,len); return; }; len=0;
  };
  virtual int print(char* buf,int len) const {
    if(cs) return cs->print(buf,len);
    if(len>0) buf[0]=0; return 0;
  };
  typedef enum {
    none,
    unknown,
    cksum,
    md5
  } type;
  static type Type(const char* crc) {
    if(!crc) return none;
    if(!crc[0]) return none;
    const char* p = strchr(crc,':');
    if(!p) {
      p=crc+strlen(crc);
      int i;
      for(i = 0;crc[i];i++) if(!isdigit(crc[i])) break;
      if(!(crc[i])) return cksum;
    };
    if(((p-crc)==5) && (strncasecmp(crc,"cksum",5)==0)) return cksum;
    if(((p-crc)==3) && (strncasecmp(crc,"md5",3)==0)) return md5;
    return unknown;
  };
  void operator=(const char* type) {
    if(cs) delete cs; cs=NULL;
    if(!type) return;
    if(strncasecmp("cksum",type,5) == 0) { cs=new CRC32Sum; return; };
    if(strncasecmp("md5",type,3) == 0) { cs=new MD5Sum; return; };
  };
  operator bool(void) { return (cs != NULL); };
};

